function Bit_String = FlexRay_A2D_Filter(filename,BP,BM,Horizon) 
 
% Opens and reads the sample frequency
file = fopen ( filename );
if file ~= -1
  fgetl ( file );
  time_delimiter = fgetl ( file );
  fclose ( file );
else 
    error('File not found')
end
time_delimiter = strtok(time_delimiter);

% Read Sample Data 
M=dlmread(filename,'\t',3,0);
t = M(:,1);

subplot(2,2,1)
plot(M(:,1),M(:,2),M(:,1),M(:,3))
title('Analouge FlexRay Samples')
xlim([M(1,1),M(end,1)])
legend('BM','BP')
ylabel('Voltage (V)')
xlabel(['time ',time_delimiter])

% Bus States
High = 3.3;
Idle = 2.6;
Low = 1.8;

% Bus State tolerances
L_1_T = [0.6 2];
L_0_T = [-2 -0.6];
Idle_T = [-0.1 0.1]; % Experimental 
Upper = 2;
Lower = 1;

%V_Bus 
V_Bus(:) = M(:,BP) - M(:,BM);
S_i = 1;

% Classifying V_Bus with Voltage Thresholds
for i = 1:size(V_Bus,2)

    if (V_Bus(i) >= L_1_T(Lower)) && (V_Bus(i) <= L_1_T(Upper))
        V_Bus(i) = High;   
    elseif (V_Bus(i) >= L_0_T(Lower)) && (V_Bus(i) <= L_0_T(Upper))
        V_Bus(i) = Low;
    elseif (V_Bus(i) >= Idle_T(Lower)) && (V_Bus(i) <= Idle_T(Upper))
       V_Bus(i) = Idle;
    else
        Settling(S_i) = i;
        S_i = S_i +1;
    end
end

Fixed_Settling = zeros(1,size(Settling,2));
Fixed_Idle = 0; 

% Average Threshold.
Avg_T = 2;

% Classifying the V_bus with Averaging
for i=2:size(V_Bus,2)-Horizon
    
    if V_Bus(i) ~= V_Bus(i-1) 
        avg = mean(V_Bus(i:i+Horizon));
        
        if any(i == Settling) 
            Fixed_Settling = Fixed_Settling + (i == Settling);
        elseif V_Bus(i) == Idle 
            Fixed_Idle = Fixed_Idle + 1; 
        end
        
        if V_Bus(i) ~= V_Bus(i+1)    
            if avg >= Avg_T 
                V_Bus(i) = High;    
            elseif avg < Avg_T
                V_Bus(i) = Low;
            end
        else
            if V_Bus(i+1) == Idle
            V_Bus(i) = Idle;
            end
        end 
    end
end

% Digitising the FlexRay Signal
diff_V_Bus = [diff(V_Bus), 0]; 
Bus_Signal(1,:) = t(diff_V_Bus ~= 0);
Bus_Signal(2,:) = diff_V_Bus( diff_V_Bus ~= 0);

for i=1:size(Bus_Signal,2)-1
    T(i)= Bus_Signal(1,i+1)-Bus_Signal(1,i);
end

Sample_Rate = (t(end) - t (1))/size(t,1);

if strcmp(time_delimiter,'(us)')
    T = T.*1000;
    Sample_Rate = Sample_Rate * 1000;
elseif strcmp(time_delimiter,'(ms)')
    T = T.*1000000;
    Sample_Rate = Sample_Rate * 1000000;
else
    disp(sprintf( 'Time delimiter: %s', time_delimiter ))
    error('Time_Delimiter is not yet added')
end

for i=1:size(Bus_Signal,2)
    
    if Bus_Signal(2,i) == Low-Idle 
        Bus_Signal(2,i)= 0;
    
    elseif Bus_Signal(2,i) == Idle-Low    
        Bus_Signal(2,i)= 1;
    
    elseif Bus_Signal(2,i) == Low-High
        Bus_Signal(2,i)= 0;
        
    elseif Bus_Signal(2,i) == High-Low
        Bus_Signal(2,i)= 1;
        
    elseif Bus_Signal(2,i) == High-Idle
        Bus_Signal(2,i)= 1;
        
    elseif Bus_Signal(2,i) == Idle-High
        Bus_Signal(2,i)= 1;
        
    else
        Bus_Signal(2,i) = -2;
    end
end

% Calculate the Logical Bit Values
Bit_Values(1,:) = Bus_Signal(2,1:end-1); 
Bit_Values(2,:) = round(T/100);   
Bit_Signal = repmat(Bit_Values(1,1), Bit_Values(2,1),1);
for i=2:size(Bit_Values,2)
    Bit_Signal = [Bit_Signal; repmat(Bit_Values(1,i), Bit_Values(2,i),1)];
end

for k = 1:size(V_Bus,2)
    if V_Bus(k) == Idle
        V_Bus(k)= High;
    end
end

N_Bit = size(Bit_Signal,1);

Bit_String = num2str(Bit_Signal');
Bit_String(isspace(Bit_String)) = '';

subplot(2,2,2)
plot(t,V_Bus,t,diff_V_Bus)
title('V_{Bus} VS dV_{Bus}')
xlim([t(1),t(end)])
ylabel('Voltage (V)')
xlabel(['time ',time_delimiter])
legend('V_{Bus}',' dV_{Bus}')

subplot(2,2,[3 4])
plot(t,V_Bus)
title('Digital Representation of Bus Signal')
ylabel('Voltage (V)')
xlabel(['time ',time_delimiter])
legend('V_{Bus}')

% Warning of 
if size(unique(V_Bus),2)>= 3 % Error check
    warning('V_Bus contains undefined bus states')
elseif nnz(Fixed_Settling)~=size(Settling,2)
    warning('Some Settling samples are not fixed')
end

% Display Values and Results
disp(sprintf( 'Number of Bits in Sample: %d', N_Bit ) )
disp(sprintf( 'Sample Rate: %d ns', Sample_Rate ) )

end
