function message = FlexRay_MessageParser(Bit_String) 

CID = '11111111111';
BSS = '10';
FES = '01';

FlexRay_Messages = strsplit(Bit_String,[CID '0']);
FlexRay_Messages = char(FlexRay_Messages);

% For Each Message in the Sample Data:
for j = 1: size(FlexRay_Messages',2)
   
    message(j).ERROR_TSS = 0;
    message(j).ERROR_FSS = 0;
    message(j).ERROR_BSS = 0;
    message(j).ERROR_FES = 0;
    
    % Correct for the CID Separation
    if j == 1
        message(j).data = strcat(strtrim(FlexRay_Messages(j,:)),CID);

    elseif j == size(FlexRay_Messages',2)
        message(j).data = strcat('0',strtrim(FlexRay_Messages(j,:)));
    else
        message(j).data = strcat('0',strtrim(FlexRay_Messages(j,:)),CID);
    end

    if j ~= size(FlexRay_Messages',2)
        temp = strsplit(message(j).data,['01' CID]);
        temp(1) = strcat(temp(1),'01');
        temp(2) = strcat(CID,temp(2));
        temp = char(temp);

        message(j).data = strtrim(temp(1,:));
        message(j).ChannelIdle_bits = size(strtrim(temp(2,:)),2);
        clear temp
    else
        message(j).ChannelIdle_bits = 0;

        while (strcmp(message(j).data(end-1:end),'01')== 0)

            message(j).data = message(j).data(1:end-1);
        end
    end

    %Calculate DTS
    if strcmp(message(j).data(end-10:end-1),'0000000000')

        temp_data = message(j).data;

        if strcmp(temp_data(end),'1')
            temp_data = temp_data(1:end-1);
        end

        DTS = 0;
        while strcmp(temp_data(end),'0')
            DTS = 1 + DTS;
            temp_data = temp_data(1:end-1);
        end    
        message(j).DTSlength_bits = DTS;

        message(j).data = temp_data;

    else
        message(j).DTSlength_bits = 0;
    end

    message_data = message(j).data;

    % Transmission Start Sequence (TSS)
    for i=1:length(message(j).data)
        if strcmp(message(j).data(i:i+2),'011')
            message(j).info.TSS = message(j).data(1:i);
            break;
        end
    end
    message(j).info.TSS_bits =  size(message(j).info.TSS,2);

    % Frame Start Sequence (FSS)
    message(j).info.FSS = message(j).data(i+1);
    if ~strcmp(message(j).info.FSS,'1')
        message(j).ERROR_FSS = 'YES';
    end

    % First Byte Start Sequence (BSS)
    if ~strcmp(message(j).data(i+2:i+3),BSS)
        message(j).ERROR_BSS = 'YES';
    end

    % Frame End Sequence (FES)
    message(j).info.FES = message(j).data(end-1:end);
    if  ~strcmp(message(j).info.FES,FES)
        message(j).ERROR_FES = 'YES';
    end

    % All 8N1 Type Byte In Frame
    message(j).data = message(j).data(i+2:end-2);

    message(j).length = length(message(j).data); 

    for i=1:message(j).length/10

        if ~strcmp(message(j).data(1:2),BSS)
            message(j).ERROR_BSS = 'YES';
        end
        
        BSS_Array(i,:) = message(j).data(1:2);
        byte(i,:) = message(j).data(3:10); 
        message(j).data = message(j).data(11:end);
    end
    
    message(j).info.BSS = BSS_Array;
    clear message(j).data BSS_Array
    message(j).byte = byte;
    clear byte
    
    % Header Portion of Frame
    message(j).Header = dec2hex(bin2dec(message(j).byte(1:5,:)));

    % Status Portion
    message(j).Status = strcat('0x '...
        ,dec2hex(bin2dec(message(j).byte(1,1:5))));

    message(j).Reserved_bit = bitand(uint8(hex2dec...
        (message(j).Status(3:end))),hex2dec('10')) == hex2dec('10');
    message(j).Payload_Preamble_bit = bitand(uint8(hex2dec...
        (message(j).Status(3:end))),hex2dec('08')) == hex2dec('08');
    message(j).Null_Frame_bit = bitand(uint8(hex2dec...
        (message(j).Status(3:end))),hex2dec('04')) == hex2dec('04');
    message(j).Sync_Frame_bit = bitand(uint8(hex2dec...
        (message(j).Status(3:end))),hex2dec('02')) == hex2dec('02');
    message(j).Startup_Frame_bit = bitand(uint8(hex2dec...
        (message(j).Status(3:end))),hex2dec('01')) == hex2dec('01');
   
    % Frame ID (11 bit)
    message(j).ID = strcat('0x ',dec2hex(bin2dec(strcat...
        (message(j).byte(1,6:8),message(j).byte(2,:)))));

    % Payload Lenght, DLC (7 bit) 
    message(j).Payload_Length = dec2hex(bin2dec...
        (message(j).byte(3,1:7)));

    % Header CRC (11 bit)
    message(j).Header_CRC = strcat('0x ',dec2hex(bin2dec(strcat(message(j).byte(3,8),...
        message(j).byte(4,:),message(j).byte(5,1:2)))));

    % Cycle count (6 bit)
    message(j).Cycle_Count = strcat('0x ',dec2hex(bin2dec(message(j).byte(5,3:8))));

    % Frame Payload Portion
    for i=1:2*hex2dec(message(j).Payload_Length)
        Payload(i,:) = message(j).byte(i+5,:);
    end
    
    message(j).Payload = dec2hex(bin2dec(Payload));
    clear Payload
    
    % Trailer Portion of Frame
    message(j).Trailer = dec2hex(bin2dec(strcat(message(j).byte(end-2,:)...
        ,message(j).byte(end-1,:)...
        ,message(j).byte(end,:))));
    message(j).data = message_data;
    
    message(j).Payload_Length = strcat('0x ',...
        char(message(j).Payload_Length));
    
    % Calcultaing Header CRC, Header_CRC()  
    message(j).Correct_HeaderCRC = strcmp(message(j).Header_CRC(3:end)...
    ,Header_CRC(message(j).Header));
    
    % Calcultaing CRC Trailer Channel A, Trailer_CRC()
    message(j).Trailer_CRC_A = strcmp(message(j).Trailer,...
        Trailer_CRC(message(j).Header,message(j).Payload...
        ,'A'));
    
    % Calcultaing CRC Trailer Channel B, Trailer_CRC()
    message(j).Trailer_CRC_B = strcmp(message(j).Trailer,...
        Trailer_CRC(message(j).Header,message(j).Payload...
        ,'B'));
    
end

% Fixs order in message variable
message = orderfields(message, [12,1:11,13:26]);
message = orderfields(message, [1,18,2:17,19:26]);
message = orderfields(message, [1:2,19,3:18,20:26]);
message = orderfields(message, [1:3,20,4:19,21:26]);
message = orderfields(message, [1:4,21,5:20,22:26]);
message = orderfields(message, [1:5,22,6:21,23:26]);
message = orderfields(message, [1:6,23,7:22,24:26]);
message = orderfields(message, [1:7,13,8:12,14:26]);
message = orderfields(message, [1:8,14,9:13,15:26]);
message = orderfields(message, [1:9,15,10:14,16:26]);
message = orderfields(message, [1:7,24,8:23,25:26]);
message = orderfields(message, [1:8,25,9:24,26]);
message = orderfields(message, [1:9,26,10:25]);
message = orderfields(message, [1:7,22,8:21,23:26]);
message = orderfields(message, [1:8,23,9:22,24:26]);
message = orderfields(message, [1:9,24,10:23,25:26]);
message = orderfields(message, [1:10,25,11:24,26]);
message = orderfields(message, [1:11,26,12:25]);

end